@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.2   2011/06/20
+ * Version: 1.8.2+   2011/07/02
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License v2 as published by the
@@ -57,7 +57,12 @@ static void show_result2(int result)
 			else
 				printf("FAILED: %s\n", strerror(errno));
 		} else {
-			printf("BUG!(%d)\n", result);
+			char c;
+			if (recv(result, &c, 1, MSG_DONTWAIT) == EOF &&
+			    errno == EPERM)
+				printf("OK: Permission denied.\n");
+			else
+				printf("BUG!(%d)\n", result);
 		}
 	} else {
 		if (result != EOF)
